package com.example.KafkaProducer;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.apache.kafka.clients.producer.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.*;
import org.springframework.stereotype.*;
import reactor.core.publisher.Flux;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderOptions;
import reactor.kafka.sender.SenderRecord;
@Service
public class ReactorKafkaProducer {

    private static final Logger log = LoggerFactory.getLogger(ReactorKafkaProducer.class.getName());


    private final KafkaSender<String, String> sender;
    @Value("${spring.kafka.producer.topic-name}")
    private final String kafkaTopic;
    private final SimpleDateFormat dateFormat;

    public ReactorKafkaProducer(@Value("${spring.kafka.producer.topic-name}") String kafkaTopic, KafkaSender<String, String> sender) {
        this.sender = sender;
        this.kafkaTopic = kafkaTopic;
        dateFormat = new SimpleDateFormat("HH:mm:ss:SSS z dd MMM yyyy");
    }

   public void sendMessages() throws InterruptedException {
        sender.send(Flux.just("John","jenny", "Janu")
                .map(i -> SenderRecord.create(new ProducerRecord<>(kafkaTopic, "names", i), i)))
                .doOnError(e -> log.error("Send failed", e))
                .subscribe(r -> {
                    RecordMetadata metadata = r.recordMetadata();
                    System.out.printf("Message %s sent successfully, topic-partition=%s - %d offset=%d timestamp=%s\n",
                            r.correlationMetadata(),
                            metadata.topic(),
                            metadata.partition(),
                            metadata.offset(),
                            dateFormat.format(new Date(metadata.timestamp())));
                });
    }

    public void close() {
        sender.close();
    }
}
