package com.example.config;

import com.example.WebClient.*;
import org.apache.kafka.clients.producer.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.*;
import reactor.kafka.sender.*;

import java.text.*;
import java.util.*;

@Configuration
public class KafkaConfig {

    private static final Logger LOGGER = LoggerFactory.getLogger(KafkaConfig.class);
    private final String clientId;
    private final String bootStrapServers;
    private final String keySerializer;
    private final String valueSerializer;
    private final String ackSetting;

    public KafkaConfig(@Value("${spring.kafka.producer.client-id}") String clientId,
                       @Value("${spring.kafka.producer.bootstrap-servers}") String bootStrapServers,
                       @Value("${spring.kafka.producer.key-serializer}") String keySerializer,
                       @Value("${spring.kafka.producer.value-serializer}") String valueSerializer,
                       @Value("${spring.kafka.producer.ack-setting}") String ackSetting) {
        this.clientId = clientId;
        this.bootStrapServers = bootStrapServers;
        this.keySerializer = keySerializer;
        this.valueSerializer = valueSerializer;
        this.ackSetting = ackSetting;
    }

    @Bean
    public KafkaProducer<String, String> getKafkaProducer() {
        LOGGER.info("Initializing Kafka Producer");
        Properties properties = new Properties();
        properties.put("client.id", clientId);
        properties.put("bootstrap.servers", bootStrapServers);
        properties.put("acks", ackSetting);
        properties.put("key.serializer", keySerializer);
        properties.put("value.serializer", valueSerializer);
        return new KafkaProducer<>(properties);
    }

    @Bean
    public KafkaSender getKafkaSender() {
        final KafkaSender<Integer, String> sender;
        LOGGER.info("Initializing Kafka Sender");
        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootStrapServers);
        props.put(ProducerConfig.CLIENT_ID_CONFIG, clientId);
        props.put(ProducerConfig.ACKS_CONFIG, ackSetting);
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, keySerializer);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, valueSerializer);

        SenderOptions<Integer, String> senderOptions = SenderOptions.create(props);

        return KafkaSender.create(senderOptions);
    }
}
