package com.example.service;

import org.apache.kafka.clients.producer.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import reactor.core.publisher.*;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

@Service
public class CopientKafkaService {

    private static final Logger LOGGER = LoggerFactory.getLogger(CopientKafkaService.class);
    private final KafkaProducer<String, String> kafkaProducer;
    private final String kafkaTopic;

    public CopientKafkaService(KafkaProducer<String, String> kafkaProducer, @Value("${spring.kafka.producer.topic-name}") String kafkaTopic) {
        this.kafkaProducer = kafkaProducer;
        this.kafkaTopic = kafkaTopic;
    }

    public void sendDataToKafkaTopic(String jsonData) throws ExecutionException, InterruptedException {
        LOGGER.info("Sending data to Kafka Topic...");
        ProducerRecord<String, String> data = new ProducerRecord<>(kafkaTopic, "copientCsvData", jsonData);
        Future<RecordMetadata> future = kafkaProducer.send(data);
        RecordMetadata metadata = future.get();
        LOGGER.info("Getting Response Metadata == {}", metadata.toString());
        kafkaProducer.flush();
    }
}
