package com.example.service;

import com.example.util.JsonMappingUtils;
import com.example.util.JsonUtils;
import org.springframework.stereotype.Component;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.*;

@Component
public class CsvReader {

    private final CopientKafkaService kafkaService;

    public CsvReader(CopientKafkaService kafkaService) {
        this.kafkaService = kafkaService;
    }

    public String convert(String dataFilePath, String mappingFilePath) throws IOException {
        StringBuffer json = new StringBuffer();
        InputStream inputStream = new FileInputStream(dataFilePath);
        String header;
        JsonUtils.addListPrefixToJson(json);
        try (Scanner fileScanner = new Scanner(inputStream, StandardCharsets.UTF_8.name())) {
            header = fileScanner.hasNextLine() ? fileScanner.nextLine() : "";
            if (!header.isEmpty()) {
                JsonMappingUtils.getInstance(mappingFilePath, header);
                while (fileScanner.hasNextLine()) {
                    String dataLine = fileScanner.nextLine();
                    if (dataLine.length() > header.split(JsonUtils.COMMA).length) {
                        String output = apply(dataLine);
                       // kafkaService.sendDataToKafkaTopic(output);
                        json.append(output).append(",");
                        json.deleteCharAt(json.length() - 1);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
//        Stream<String> csvDataList = Files.lines(Paths.get(dataFilePath), StandardCharsets.UTF_8);
//        JsonMappingUtils.getInstance(mappingFilePath, header);
//        JsonUtils.addListPrefixToJson(json);
//        Optional<String> finalOutput = csvDataList
//                .skip(1)
//                .filter(s -> !s.isEmpty())
//                .parallel()
//                .map(new CsvDataReader())
//                .reduce((jsonObj1, jsonObj2) -> jsonObj1 + "," + jsonObj2);
//        finalOutput.ifPresent(json::append);
        JsonUtils.addListSuffixToJson(json);
        return json.toString();
    }

    private String apply(String csvRowData) throws Exception {
        StringBuilder finalOutput = new StringBuilder();
        Optional<Map<String, Integer>> jsonVarToCsvDataIndexOptional = Optional.of(JsonMappingUtils.getDefaultInstance().getJsonVarToCsvDataIndex());
        jsonVarToCsvDataIndexOptional.ifPresent(jsonVarToCsvDataIndex -> beginTransformToJson(csvRowData, jsonVarToCsvDataIndex, finalOutput));
        return finalOutput.toString();
    }

    private void beginTransformToJson(String csvDataLine, Map<String, Integer> jsonVarToCsvDataIndex, StringBuilder finalOutput) {
        String[] splitData = csvDataLine.split(",");
        if (jsonVarToCsvDataIndex.keySet().size() != splitData.length) {
            splitData = fixDoubleQuoteCsvFormatIssue(splitData);
        }
        List<String> correctedData = removeDoubleQuoteFromCsvData(splitData);
        if (jsonVarToCsvDataIndex.size() - splitData.length > 0) {
            fixCsvMissingData(correctedData, jsonVarToCsvDataIndex.size() - splitData.length);
        }
        StringBuffer jsonBody = new StringBuffer();
        jsonVarToCsvDataIndex.forEach((key, value) -> JsonUtils.jsonVariable(jsonBody, key.trim(), correctedData.get(value).trim()));
        jsonBody.deleteCharAt(jsonBody.length() - 1);
        finalOutput.append(JsonUtils.addObjectBodyToJson(jsonBody));
        finalOutput.deleteCharAt(finalOutput.length() - 1);
    }

    private String[] fixDoubleQuoteCsvFormatIssue(String[] splitData) {
        String[] correctedCsvData = new String[0];
        for (int i = 0; i < splitData.length; i++) {
            StringBuilder columnData = new StringBuilder(splitData[i]);
            if (columnData.toString().startsWith("\"")) {
                int count = 1;
                while (!columnData.toString().endsWith("\"")) {
                    columnData.append(",").append(splitData[i + count]);
                    count++;
                }
                splitData[i] = columnData.toString();
                correctedCsvData = Arrays.copyOf(splitData, splitData.length - count + 1);
            }

        }
        return correctedCsvData.length == 0 ? splitData : correctedCsvData;
    }

    private List<String> removeDoubleQuoteFromCsvData(String[] splitData) {
        List<String> correctedDataList = new ArrayList<>();
        for (String columnData : splitData) {
            correctedDataList.add(columnData.replaceAll("\"", ""));
        }
        return correctedDataList;
    }

    private void fixCsvMissingData(List<String> csvData, int missingColumnDataCount) {
        while (missingColumnDataCount >= 0) {
            csvData.add("");
            missingColumnDataCount--;
        }
    }
}
