package com.example.util;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JsonMappingUtils {

    private transient Map<String, Integer> jsonVarToCsvDataIndex = new HashMap<>();
    private static JsonMappingUtils jsonMappingUtils = null;

    private JsonMappingUtils() {

    }

    private JsonMappingUtils(String mappingFilePath, String csvHeaderInfo) {
        populateJsonVarVsIndexMap(mappingFilePath, csvHeaderInfo);
    }

    public static JsonMappingUtils getInstance(String mappingFilePath, String csvHeaderInfo) {
        if (jsonMappingUtils == null) {
            synchronized (JsonMappingUtils.class) {
                if (jsonMappingUtils == null) {
                    jsonMappingUtils = new JsonMappingUtils(mappingFilePath, csvHeaderInfo);
                }
            }
        }
        return jsonMappingUtils;
    }

    public static JsonMappingUtils getDefaultInstance() throws Exception {
        if (jsonMappingUtils == null) {
            synchronized (JsonMappingUtils.class) {
                if (jsonMappingUtils == null) {
                    jsonMappingUtils = new JsonMappingUtils();
                    return jsonMappingUtils;
                }
            }
        } else {
            if (jsonMappingUtils.jsonVarToCsvDataIndex.isEmpty()) {
                throw new Exception("Mapping content unavailable, initialize the Mapping Util with parameterized static instance method");
            }
        }
        return jsonMappingUtils;
    }

    private void populateJsonVarVsIndexMap(String mappingFilePath, String csvHeaderInfo) {
        List<String> csvHeaderNames = Arrays.asList(csvHeaderInfo.split(","));
        Map<String, String> headerNameVsJsonVarMap = new HashMap<>();
        try {
            List<String> list = Files.readAllLines(Paths.get(mappingFilePath), StandardCharsets.UTF_8);
            list.forEach(s -> {
                String[] headerVsVar = s.split(":");
                headerNameVsJsonVarMap.put(headerVsVar[0], headerVsVar[1]);
            });
        } catch (IOException e) {
            e.printStackTrace();
        }
        headerNameVsJsonVarMap.forEach((key, value) -> {
            int headerIndex = csvHeaderNames.indexOf(key);
            jsonVarToCsvDataIndex.put(value.trim(), headerIndex);
        });
    }

    public Map<String, Integer> getJsonVarToCsvDataIndex() {
        return new HashMap<>(jsonVarToCsvDataIndex);
    }
}
